@@ -2,10 +2,13 @@
 A module for working with asyncio.
 """
 
+
 # built-in
 from asyncio import AbstractEventLoop as _AbstractEventLoop
+from asyncio import CancelledError as _CancelledError
 from asyncio import all_tasks as _all_tasks
-from asyncio import gather as _gather
+
+# from asyncio import gather as _gather
 from contextlib import suppress as _suppress
 from typing import Any as _Any
 from typing import Awaitable as _Awaitable
@@ -15,15 +18,17 @@ from typing import Optional as _Optional
 def shutdown_loop(eloop: _AbstractEventLoop) -> None:
     """Attempt to shut down an event loop."""
 
+    eloop.run_until_complete(eloop.shutdown_asyncgens())
+
     tasks = [x for x in _all_tasks(loop=eloop) if not x.done()]
     if tasks:
         # Cancel all tasks running in the event loop.
         for task in tasks:
             task.cancel()
 
-        # Wait for the task to complete.
-        with _suppress(KeyboardInterrupt):
-            eloop.run_until_complete(_gather(*tasks))
+            # Give all tasks a chance to complete.
+            with _suppress(KeyboardInterrupt, _CancelledError):
+                eloop.run_until_complete(task)
 
 
 def run_handle_interrupt(
